import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import supply_kit_pull_system.SupplyChainModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import simudyne.core.exec.runner.ModelRunner;
import simudyne.core.exec.runner.RunnerBackend;
import simudyne.core.exec.runner.definition.BatchDefinitionsBuilder;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;

public class HeadlessMain {
    private static final Logger logger = LoggerFactory.getLogger("main");

    public static void main(String[] args) {
        Map<String, Object> _inputs = getMap("");
        Map<String, Object> _globalInputs = new HashMap<>();
        String outputPath;
        String jsonInputPath;
        if (args.length == 2) {
            jsonInputPath = args[0];
            outputPath = args[1];
        } else {
            jsonInputPath = "test_sim_inputs.json";
            outputPath = "output";
        }

        String result;
        try {
            result = new String(Files.readAllBytes(Paths.get(jsonInputPath)));
            _globalInputs = new Gson().fromJson(result, new TypeToken<HashMap<String, Object>>() {
            }.getType());
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            RunnerBackend runnerBackend = RunnerBackend.create();
            ModelRunner modelRunner = runnerBackend.forModel(SupplyChainModel.class);

            long ticks = (long) Double.parseDouble(_globalInputs.get("ticks").toString());
            int n_runs = (int) Double.parseDouble(_globalInputs.get("runs").toString());
            long seed = (long) Double.parseDouble(_globalInputs.get("seed").toString());

            System.out.println(seed);
            long startTime = System.currentTimeMillis();

            modelRunner.withConfig("simudyne.core.export-path", outputPath);

            //_inputs.put("system", _globalInputs);

            BatchDefinitionsBuilder runDefinitionBuilder =
                    BatchDefinitionsBuilder.create()
                            .forRuns(n_runs)
                            .forTicks(ticks)
                            .forGeneratorSeed(seed);
            //.withInputs(_inputs);

            logger.warn("Simulation Starting...");
            modelRunner.forRunDefinitionBuilder(runDefinitionBuilder);
            modelRunner.run().awaitOutput();
            
            // Output data
            logger.warn("Simulation Complete.");

            long endTime = System.currentTimeMillis();
            logger.warn("Time for simulation = " + (double) (endTime - startTime) / 1000.0 + " s.");

            System.exit(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public static Map<String, Object> getMap(String str) {
        Map<String, Object> _map = new HashMap<>();
        if (str.isEmpty())
            return _map;

        for (String s0 : str.split(",")) {
            String[] s1 = s0.split("=");
            if (s1.length != 2)
                throw new IllegalArgumentException("Unknown input format, " + s0);

            _map.put(s1[0], s1[1]);
        }

        return _map;
    }
}