package supply_kit_pull_system.agents;

import simudyne.core.abm.Action;
import simudyne.core.abm.Agent;
import simudyne.core.annotations.Constant;
import supply_kit_pull_system.Globals;
import supply_kit_pull_system.util.AvailableStock;
import supply_kit_pull_system.util.Messages;
import supply_kit_pull_system.util.ProductDetails;
import supply_kit_pull_system.util.WarehouseOrder;

import java.util.*;

public class MarketPlace extends Agent<Globals> {

    public Map<String, ProductDetails> availableProducts;

    @Constant
    public String name;

    public Map<String, List<AvailableStock>> availableInventory = new HashMap<>();

    public static Action<MarketPlace> matchOrders =
            Action.create(MarketPlace.class, marketPlace -> {
                marketPlace.matchOrders();
            });

    public void matchOrders() {

        //Clear inventory
        availableInventory.clear();

        //Get latest inventory information and store
        getMessagesOfType(Messages.InventoryMessage.class).stream().forEach(inventoryMessage -> {
            inventoryMessage.inventory.entrySet().stream().forEach(entry -> {
                String productID = entry.getKey();
                int totalInventory = entry.getValue().size();
                if (totalInventory > 0) {
                    AvailableStock stock = new AvailableStock(inventoryMessage.getSender(), totalInventory);
                    availableInventory.computeIfAbsent(productID, k -> new ArrayList<>()).add(stock);
                }
            });
        });


        List<Messages.OrderMessage> incomingOrders = getMessagesOfType(Messages.OrderMessage.class);
        for (Messages.OrderMessage order : incomingOrders) {
            Map<String, Integer> outOfStockItems = new HashMap<>();
            Map<String, Integer> orderContents = order.marketPlaceOrderDetails.orderedProducts;
            List<WarehouseOrder> matchedOrders = new ArrayList<>();
            orderContents.entrySet().stream().forEach(purchase -> {
                String productID = purchase.getKey();
                int orderQuantity = purchase.getValue();
                List<AvailableStock> stockList = availableInventory.get(productID);
                if (stockList != null && !stockList.isEmpty()) {
                    Collections.shuffle(stockList, getPrng().getRandom());
                    Iterator<AvailableStock> stockIterator = stockList.iterator();
                    while (stockIterator.hasNext() && orderQuantity > 0) {
                        AvailableStock stock = stockIterator.next();
                        if (stock.totalStock >= orderQuantity) {
                            stock.totalStock -= orderQuantity;
                            matchedOrders.add(new WarehouseOrder(
                                    productID,
                                    orderQuantity,
                                    order.marketPlaceOrderDetails.customerID,
                                    order.marketPlaceOrderDetails.customerRegion,
                                    order.marketPlaceOrderDetails.orderDate,
                                    order.marketPlaceOrderDetails.shippingPreference,
                                    stock.warehouseID));
                            orderQuantity = 0;
                        } else {
                            orderQuantity -= stock.totalStock;
                            matchedOrders.add(new WarehouseOrder(
                                    productID,
                                    stock.totalStock,
                                    order.marketPlaceOrderDetails.customerID,
                                    order.marketPlaceOrderDetails.customerRegion,
                                    order.marketPlaceOrderDetails.orderDate,
                                    order.marketPlaceOrderDetails.shippingPreference,
                                    stock.warehouseID));
                            stock.totalStock = 0;
                            stockIterator.remove();
                        }
                    }
                    availableInventory.put(productID, stockList);
                }
                if (orderQuantity > 0) {
                    outOfStockItems.put(productID, orderQuantity);
                }
            });

            if (outOfStockItems.size() > 0) {
                send(Messages.OutOfStockMessage.class, msg -> {
                    msg.outOfStockItems = outOfStockItems;
                }).to(order.getSender());
            }

            if (matchedOrders.size() > 0) {
                matchedOrders.stream().forEach(wo -> {
                    send(Messages.WarehouseOrderMessage.class, msg -> {
                        msg.warehouseOrder = wo;
                    }).to(wo.warehouseID);
                });
            }
        }
    }
}
